package pps.core.prediction.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.BasePhotovoltaicPlantCloudService;
import pps.cloud.base.service.IBasePowerLineCloudService;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudInput;
import pps.cloud.base.service.data.base_photovoltaic_plant.GetBasePhotovoltaicPlantCloudOutput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineInput;
import pps.cloud.base.service.data.base_power_line.DynamicQueryBasePowerLineOutput;
import pps.cloud.prediction.service.IThirdDataAccessCloudService;
import pps.core.prediction.constant.ThirdPartyApiConstant;
import pps.core.prediction.entity.*;
import pps.core.prediction.mapper.*;
import pps.core.prediction.service.data.third_active_power.GetThirdActivePowerViewOutput;
import pps.core.prediction.service.data.third_current_well_condition.GetThirdCurrentWellConditionViewOutput;
import pps.core.prediction.service.data.third_daily_accumulation.GetCumulativePowerGenerationOutput;
import pps.core.prediction.service.data.third_photovoltaic_power.GetThirdPhotovoltaicPowerViewOutput;
import pps.core.prediction.utils.ServiceUtil;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.helper.XTransactionHelper;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 第三方数据接入Cloud模块
 *
 * @author ZWT
 * @date 2024/03/19 10:31
 */
@XText("第三方数据接入Cloud模块")
@XService
public class ThirdDataAccessCloudServiceImpl implements IThirdDataAccessCloudService {

    /**
     * 第三方有功功率定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdActivePowerJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_STATION_PV_DATA,
                new HashMap<>(0));
        context.getLogger().info("*** CQ_WELL_STATION_PV_DATA : {}", resultStr);
        List<GetThirdActivePowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdActivePowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        Map<String, String> collect = this.getPhotovoltaicPlantMap(context);
        List<ThirdActivePowerView> dtoList = new ArrayList<>(outputs.size());
        List<ThirdActivePowerDailyUpdateView> dtoDailyList = new ArrayList<>(outputs.size());
        DateTime now = DateTime.now();
        DateTime saveDate = DateUtil.beginOfDay(now);
        DateTime saveHour = DateUtil.beginOfHour(now);
        BigDecimal photovoltaicPower;
        BigDecimal meterPower;
        LambdaQueryWrapper<ThirdActivePowerEnt> queryWrapper = new LambdaQueryWrapper<ThirdActivePowerEnt>()
                .select(ThirdActivePowerEnt::getId);
        for (GetThirdActivePowerViewOutput o : outputs) {
            photovoltaicPower = o.getPhotovoltaicPower().abs();
            meterPower = o.getMeterPower().abs();
            dtoList.add(
                    ThirdActivePowerView.builder()
                            .stationName(o.getStationName())
                            .stationId(collect.get(o.getStationName()))
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .meterPower(meterPower)
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .inputTime(now)
                            .build()
            );
            queryWrapper.or(w -> w.eq(ThirdActivePowerEnt::getStationName, o.getStationName())
                    .eq(ThirdActivePowerEnt::getCreateDate, o.getCreateDate())
                    .eq(ThirdActivePowerEnt::getPhotovoltaicPower, o.getPhotovoltaicPower())
                    .eq(ThirdActivePowerEnt::getMeterPower, o.getMeterPower())
                    .eq(ThirdActivePowerEnt::getSystemSource, ThirdPartyApiConstant.CQ_SOURCE)
            );
            dtoDailyList.add(
                    ThirdActivePowerDailyUpdateView.builder()
                            .stationName(o.getStationName())
                            .stationId(collect.get(o.getStationName()))
                            .createDate(o.getCreateDate())
                            .photovoltaicPower(photovoltaicPower)
                            .meterPower(meterPower)
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .saveDate(saveDate)
                            .saveHour(saveHour)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            //删除重复数据
            ThirdActivePowerMapper powerMapper = context.getBean(ThirdActivePowerMapper.class);
            List<ThirdActivePowerEnt> list = powerMapper.selectList(queryWrapper);
            if (CollUtil.isNotEmpty(list)) {
                powerMapper.deleteBatchIds(list.stream()
                        .map(ThirdActivePowerEnt::getId)
                        .collect(Collectors.toList()));
            }
            ThirdActivePowerViewMapper mapper = context.getBean(ThirdActivePowerViewMapper.class);
            //保存历史数据
            mapper.batchInsertList(dtoList);
            ThirdActivePowerDailyUpdateViewMapper dailyMapper = context.getBean(ThirdActivePowerDailyUpdateViewMapper.class);
            //删除当日最新数据
            dailyMapper.deleteByCurrentDayData(dtoDailyList, saveHour);
            //保存当日最新数据
            dailyMapper.batchInsertList(dtoDailyList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方当日油井状态定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdCurrentWellConditionJob(XContext context) {
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_WELL_REAL_PV_DATA,
                new HashMap<>(0));
        context.getLogger().info("*** CQ_WELL_REAL_PV_DATA : {}", resultStr);
        List<GetThirdCurrentWellConditionViewOutput> outputs = JSON.parseArray(resultStr, GetThirdCurrentWellConditionViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        //先删后增
        List<ThirdCurrentWellConditionView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdCurrentWellConditionViewOutput o : outputs) {
            dtoList.add(
                    ThirdCurrentWellConditionView.builder()
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .wellNumber(o.getWellNumber())
                            .nextTime(o.getNextTime())
                            .wellStatus(Optional.ofNullable(o.getWellStatus()).orElse("停井"))
                            .cumulativeProduction(Optional.ofNullable(o.getCumulativeProduction()).orElse(BigDecimal.ZERO))
                            .runningTime(o.getRunningTime())
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdCurrentWellConditionViewMapper mapper = context.getBean(ThirdCurrentWellConditionViewMapper.class);
            mapper.deleteByParams(dtoList);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方光伏日发电量定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdPhotovoltaicPowerJob(XContext context) {
        Map<String, Object> param = new HashMap<>(4);
        param.put("Calc_Date", DateUtil.yesterday().toString("yyyy-MM-dd"));
        String resultStr = ServiceUtil.doPostFormCq(context,
                ThirdPartyApiConstant.CQ_STATION_DAILY_PV_DATA,
                param);
        context.getLogger().info("*** CQ_STATION_DAILY_PV_DATA : {}", resultStr);
        List<GetThirdPhotovoltaicPowerViewOutput> outputs = JSON.parseArray(resultStr, GetThirdPhotovoltaicPowerViewOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        List<ThirdPhotovoltaicPowerView> dtoList = new ArrayList<>(outputs.size());
        for (GetThirdPhotovoltaicPowerViewOutput output : outputs) {
            dtoList.add(
                    ThirdPhotovoltaicPowerView.builder()
                            .stationName(output.getStationName())
                            .createDate(output.getCreateDate())
                            .photovoltaicPower(Optional.ofNullable(output.getPhotovoltaicPower())
                                    .orElse(BigDecimal.ZERO))
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .build()
            );
        }
        return XTransactionHelper.begin(context, () -> {
            ThirdPhotovoltaicPowerViewMapper mapper = context.getBean(ThirdPhotovoltaicPowerViewMapper.class);
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /**
     * 能耗分析定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult energyConsumptionAnalysisJob(XContext context) {
        DateTime yesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        context.getLogger().info("------ EnergyConsumptionAnalysisJob yesterday:{}", yesterday.toString());
        //查昨日光伏发电量
        ThirdPhotovoltaicPowerMapper powerMapper = context.getBean(ThirdPhotovoltaicPowerMapper.class);
        List<ThirdPhotovoltaicPowerEnt> powerList = powerMapper.selectList(new LambdaQueryWrapper<ThirdPhotovoltaicPowerEnt>()
                .eq(ThirdPhotovoltaicPowerEnt::getCreateDate, yesterday)
        );
        if (CollUtil.isEmpty(powerList)) {
            context.getLogger().info("------ EnergyConsumptionAnalysisJob powerList isEmpty");
            return XServiceResult.OK;
        }
        //通过场站名查线路ID
        Map<String, String> lineMap = this.getBasePowerLineListByNames(context, powerList.stream()
                        .map(ThirdPhotovoltaicPowerEnt::getStationName)
                        .collect(Collectors.toList())).stream()
                .collect(Collectors.toMap(DynamicQueryBasePowerLineOutput::getLineName, DynamicQueryBasePowerLineOutput::getId));
        //查昨日井口耗电量
        WellheadDailyProductionSituationViewMapper mapper = context.getBean(WellheadDailyProductionSituationViewMapper.class);
        List<WellheadDailyProductionSituationView> situationList = mapper.selectSumDailyElectricityConsumption(yesterday);
        Map<String, BigDecimal> collect = situationList.stream()
                .collect(Collectors.toMap(WellheadDailyProductionSituationView::getLineId, WellheadDailyProductionSituationView::getDailyElectricityConsumption));
        //封装
        List<EnergyConsumptionAnalysisView> analysisList = new ArrayList<>(powerList.size());
        for (ThirdPhotovoltaicPowerEnt power : powerList) {
            String lineId = lineMap.get(power.getStationName());
            analysisList.add(EnergyConsumptionAnalysisView.builder()
                    .lineId(lineId)
                    .stationName(power.getStationName())
                    .powerGeneration(power.getPhotovoltaicPower())
                    .powerConsumption(collect.get(lineId))
                    .dataDate(power.getCreateDate())
                    .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                    .build());
        }
        return XTransactionHelper.begin(context, () -> {
            EnergyConsumptionAnalysisViewMapper energyMapper = context.getBean(EnergyConsumptionAnalysisViewMapper.class);
            energyMapper.batchInsertList(analysisList);
            return XServiceResult.OK;
        });
    }

    /**
     * 第三方井场日累计数据定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult}
     */
    @Override
    public XServiceResult thirdDailyAccumulationJob(XContext context) {
        String resultStr = "[\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"王11-8\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 80.8,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 3.73,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 26.51,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0.27\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山31-14\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 305.5,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 5.21,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 191.85,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 61.39\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山33-15\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 448.8,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 6.17,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 319.5,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 102.24\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山37-11\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 287.99,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 4.49,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 23,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 7.36\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山32-1\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 4.59,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午218-207X\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 20.45,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午222-210X\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 453.6,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 1.11,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 25.56,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 8.18\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午291\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 1.35,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午平231-04\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 786,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 5.96,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 40.04,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 12.81\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山34-13\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 159,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 8.04,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 27.83,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 8.91\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山35-12\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 339.6,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 3.55,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 124.11,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 39.72\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山40-11\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 462.7,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 20.51,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 99.68,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 31.91\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"山43-9\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 859.2,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 1.9,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 56.09,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 17.95\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午249-8\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 72.66,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 24.84,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 214.57,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 68.66\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午252-9\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 168.6,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 14.85,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 44.31,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 14.18\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"午平276-2\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 435.2,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 0.55,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"沙28-22\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 361.6,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 0,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"塬47-93扩\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 30.27,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"塬48-92\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 6.04,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"沙25-12\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 8.04,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"沙30-23\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 179.09,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 0,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"塬39-96\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 10.78,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"沙24-13\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 9.58,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"塬49-92\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 8.47,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"黄120\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 0,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 6.88,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t},\n" +
                "\t\t{\n" +
                "\t\t\t\"stationname\": \"塬平57-17\",\n" +
                "\t\t\t\"calC_DATE\": null,\n" +
                "\t\t\t\"pV_DAY_ELECTRIC_POWER\": 792,\n" +
                "\t\t\t\"reaL_LIQUID_DAILY\": 0,\n" +
                "\t\t\t\"poweR_CONSUME_DAILY\": 0,\n" +
                "\t\t\t\"pV_CONSUMP_ELECTRIC\": 0\n" +
                "\t\t}\n" +
                "\t]";
        context.getLogger().info("*** CQ_GROUP_REAL_PV_DATA : {}", resultStr);
        List<GetCumulativePowerGenerationOutput> outputs = JSON.parseArray(resultStr, GetCumulativePowerGenerationOutput.class);
        if (CollUtil.isEmpty(outputs)) {
            return XServiceResult.OK;
        }
        //取电站
        Map<String, String> plantMap = this.getPhotovoltaicPlantMap(context);
        DateTime now = DateUtil.date();
        DateTime beginOfDay = DateUtil.beginOfDay(now);
        //封装数据
        List<ThirdDailyAccumulationView> dtoList = new ArrayList<>(outputs.size());
        LambdaQueryWrapper<ThirdDailyAccumulationEnt> queryWrapper = new LambdaQueryWrapper<ThirdDailyAccumulationEnt>();
        for (GetCumulativePowerGenerationOutput o : outputs) {
            dtoList.add(
                    ThirdDailyAccumulationView.builder()
                            .saveDate(beginOfDay)
                            .stationId(plantMap.get(o.getStationName()))
                            .stationName(o.getStationName())
                            .createDate(now)
                            .photovoltaicPower(o.getPhotovoltaicPower())
                            .dailyElectricityConsumption(o.getDailyElectricityConsumption())
                            .dailyLiquidProduction(o.getDailyLiquidProduction())
                            .inPlaceConsumption(o.getInPlaceConsumption())
                            .systemSource(ThirdPartyApiConstant.CQ_SOURCE)
                            .build()
            );
            //组装删除条件
            queryWrapper.or(w -> w.eq(ThirdDailyAccumulationEnt::getStationName, o.getStationName())
                    .eq(ThirdDailyAccumulationEnt::getSaveDate, beginOfDay)
                    .eq(ThirdDailyAccumulationEnt::getSystemSource, ThirdPartyApiConstant.CQ_SOURCE)
            );
        }
        return XTransactionHelper.begin(context, () -> {
            //先删除
            ThirdDailyAccumulationMapper bean = context.getBean(ThirdDailyAccumulationMapper.class);
            bean.delete(queryWrapper);
            ThirdDailyAccumulationViewMapper mapper = context.getBean(ThirdDailyAccumulationViewMapper.class);
            //保存当日最新数据
            mapper.batchInsertList(dtoList);
            return XServiceResult.OK;
        });
    }

    /*----------------------------------- private -----------------------------------*/

    /**
     * 获取光伏电站Map
     *
     * @param context 上下文
     * @return {@link Map}<{@link String}, {@link String}>
     */
    private Map<String, String> getPhotovoltaicPlantMap(XContext context) {
        XListResult<GetBasePhotovoltaicPlantCloudOutput> result = context.getBean(BasePhotovoltaicPlantCloudService.class)
                .getBasePhotovoltaicPlantList(context, new GetBasePhotovoltaicPlantCloudInput());
        return result.getResult().stream()
                .collect(Collectors.toMap(GetBasePhotovoltaicPlantCloudOutput::getStationName, GetBasePhotovoltaicPlantCloudOutput::getId));
    }

    /**
     * 查询线路列表通过线路名称
     *
     * @param context   上下文
     * @param lineNames ou-id列表
     * @return {@link List}<{@link DynamicQueryBasePowerLineOutput}>
     */
    private List<DynamicQueryBasePowerLineOutput> getBasePowerLineListByNames(XContext context, List<String> lineNames) {
        IBasePowerLineCloudService basePowerLineCloudService = context.getBean(IBasePowerLineCloudService.class);
        XListResult<DynamicQueryBasePowerLineOutput> lineList = basePowerLineCloudService.getBasePowerLineList(context, DynamicQueryBasePowerLineInput.builder()
                .lineNames(lineNames)
                .build());
        lineList.throwIfFail();
        return lineList.getResult();
    }
}
